package com.ruoyi.system.controller.user;

import com.ruoyi.common.core.domain.R;
import com.ruoyi.common.core.utils.ServletUtils;
import com.ruoyi.common.core.utils.StringUtils;
import com.ruoyi.common.core.utils.poi.ExcelUtil;
import com.ruoyi.common.core.web.controller.BaseController;
import com.ruoyi.common.core.web.domain.AjaxResult;
import com.ruoyi.common.core.web.page.TableDataInfo;
import com.ruoyi.common.log.annotation.Log;
import com.ruoyi.common.log.enums.BusinessType;
import com.ruoyi.common.security.annotation.InnerAuth;
import com.ruoyi.common.security.annotation.RequiresPermissions;
import com.ruoyi.common.security.service.TokenService;
import com.ruoyi.common.security.utils.SecurityUtils;
import com.ruoyi.system.api.domain.SysRole;
import com.ruoyi.system.api.domain.SysUser;
import com.ruoyi.system.api.model.LoginUser;
import com.ruoyi.system.enums.BussLogEnum;
import com.ruoyi.system.service.ISnTemplatePermsService;
import com.ruoyi.system.service.ISysBussLogService;
import com.ruoyi.system.service.ISysDeptService;
import com.ruoyi.system.service.ISysPermissionService;
import com.ruoyi.system.service.ISysPostService;
import com.ruoyi.system.service.ISysRoleService;
import com.ruoyi.system.service.ISysUserService;
import java.io.IOException;
import java.util.Iterator;
import java.util.List;
import java.util.Objects;
import java.util.Set;
import java.util.stream.Collectors;
import javax.servlet.http.HttpServletResponse;
import org.apache.commons.lang3.ArrayUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

@RestController
@RequestMapping({"/agent"})
public class SysAgentController extends BaseController {
   @Autowired
   private ISysUserService userService;
   @Autowired
   private ISysRoleService roleService;
   @Autowired
   private ISysDeptService deptService;
   @Autowired
   private ISysPostService postService;
   @Autowired
   private ISysPermissionService permissionService;
   @Autowired
   private ISnTemplatePermsService templatePermsService;
   @Autowired
   private TokenService tokenService;
   @Autowired
   private ISysBussLogService sysBussLogService;

   @RequiresPermissions({"system:agent:list"})
   @GetMapping({"/list"})
   public TableDataInfo list(SysUser user) {
      if (user.getParams().containsKey("isPage")) {
         if ("true".equalsIgnoreCase(user.getParams().get("isPage").toString())) {
            this.startPage();
         }
      } else {
         this.startPage();
      }

      user.setUserType("agent");
      List<SysUser> list = this.userService.selectUserList(user);
      return this.getDataTable(list);
   }

   @GetMapping({"/selectList"})
   public AjaxResult selectList(SysUser user) {
      user.setUserType("agent");
      return AjaxResult.success(this.userService.selectList(user));
   }

   @Log(
      title = "代理管理",
      businessType = BusinessType.EXPORT
   )
   @RequiresPermissions({"system:agent:export"})
   @PostMapping({"/export"})
   public void export(HttpServletResponse response, SysUser user) {
      List<SysUser> list = this.userService.selectUserList(user);
      ExcelUtil<SysUser> util = new ExcelUtil(SysUser.class);
      util.exportExcel(response, list, "代理数据");
   }

   @Log(
      title = "代理管理",
      businessType = BusinessType.IMPORT
   )
   @RequiresPermissions({"system:agent:import"})
   @PostMapping({"/importData"})
   public AjaxResult importData(MultipartFile file, boolean updateSupport) throws Exception {
      ExcelUtil<SysUser> util = new ExcelUtil(SysUser.class);
      List<SysUser> userList = util.importExcel(file.getInputStream());
      String operName = SecurityUtils.getUsername();
      String message = this.userService.importUser(userList, updateSupport, operName);
      return this.success(message);
   }

   @PostMapping({"/importTemplate"})
   public void importTemplate(HttpServletResponse response) throws IOException {
      ExcelUtil<SysUser> util = new ExcelUtil(SysUser.class);
      util.importTemplateExcel(response, "代理数据");
   }

   @InnerAuth
   @GetMapping({"/info/{username}"})
   public R<LoginUser> info(@PathVariable("username") String username) {
      SysUser sysUser = this.userService.selectUserByUserName(username);
      if (StringUtils.isNull(sysUser)) {
         return R.fail("代理名或密码错误");
      } else {
         Set<String> roles = this.permissionService.getRolePermission(sysUser);
         Set<String> permissions = this.permissionService.getMenuPermission(sysUser);
         LoginUser sysUserVo = new LoginUser();
         sysUserVo.setSysUser(sysUser);
         sysUserVo.setRoles(roles);
         sysUserVo.setPermissions(permissions);
         return R.ok(sysUserVo);
      }
   }

   @GetMapping({"getInfo"})
   public AjaxResult getInfo() {
      SysUser user = this.userService.selectUserById(SecurityUtils.getUserId());
      Set<String> roles = this.permissionService.getRolePermission(user);
      Set<String> permissions = this.permissionService.getMenuPermission(user);
      AjaxResult ajax = AjaxResult.success();
      ajax.put("user", user);
      ajax.put("roles", roles);
      ajax.put("permissions", permissions);
      return ajax;
   }

   @RequiresPermissions({"system:agent:query"})
   @GetMapping({"/", "/{userId}"})
   public AjaxResult getInfo(@PathVariable(value = "userId",required = false) Long userId) {
      AjaxResult ajax = AjaxResult.success();
      List<SysRole> roles = this.roleService.selectRoleAll();
      ajax.put("roles", roles.stream().filter((r) -> {
         return r.getRoleKey().equals("agent");
      }).collect(Collectors.toList()));
      ajax.put("posts", this.postService.selectPostAll());
      if (StringUtils.isNotNull(userId)) {
         SysUser sysUser = this.userService.selectUserById(userId);
         ajax.put("data", sysUser);
         ajax.put("postIds", this.postService.selectPostListByUserId(userId));
         ajax.put("roleIds", sysUser.getRoles().stream().map(SysRole::getRoleId).collect(Collectors.toList()));
      }

      return ajax;
   }

   @RequiresPermissions({"system:agent:add"})
   @Log(
      title = "代理管理",
      businessType = BusinessType.INSERT
   )
   @PostMapping
   public AjaxResult add(@Validated @RequestBody SysUser user) {
      if (!this.userService.checkUserNameUnique(user)) {
         return this.error("新增代理'" + user.getUserName() + "'失败，登录账号已存在");
      } else if (StringUtils.isNotEmpty(user.getPhonenumber()) && !this.userService.checkPhoneUnique(user)) {
         return this.error("新增代理'" + user.getUserName() + "'失败，手机号码已存在");
      } else if (StringUtils.isNotEmpty(user.getEmail()) && !this.userService.checkEmailUnique(user)) {
         return this.error("新增代理'" + user.getUserName() + "'失败，邮箱账号已存在");
      } else {
         LoginUser loginUser = this.tokenService.getLoginUser(ServletUtils.getRequest());
         SysUser curUser = this.userService.selectUserById(loginUser.getUserid());
         if (curUser.getServerLimit() < user.getServerLimit()) {
            return this.error("您当前最多可分配" + curUser.getServerLimit() + "个服务数量");
         } else if (StringUtils.isEmpty(curUser.getCloakId())) {
            return this.error("CloakId未配置，请联系管理员");
         } else {
            user.setCloakId(curUser.getCloakId());
            user.setUserType("agent");
            user.setCreateBy(SecurityUtils.getUserId().toString());
            user.setCreateName(SecurityUtils.getLoginUser().getUsername());
            user.setPassword(SecurityUtils.encryptPassword(user.getPassword()));
            user.setDealerId(SecurityUtils.getUserId());
            user.setDealerName(SecurityUtils.getLoginUser().getUsername());
            int num = this.userService.insertUser(user);
            if (num > 0) {
               this.sysBussLogService.insertSysBussLog(BussLogEnum.AddAgent.getType(), user.getUserId(), user.getUserName(), "添加代理 " + user.getUserName());
            }

            if (user.getParams().containsKey("templateIds")) {
               try {
                  List<Integer> templateIds = (List)user.getParams().get("templateIds");
                  this.templatePermsService.updateUserPerms((List)templateIds.stream().map(Integer::longValue).collect(Collectors.toList()), user.getUserId());
               } catch (Exception var6) {
               }
            }

            return this.toAjax(num);
         }
      }
   }

   @RequiresPermissions({"system:agent:edit"})
   @Log(
      title = "代理管理",
      businessType = BusinessType.UPDATE
   )
   @PutMapping
   public AjaxResult edit(@Validated @RequestBody SysUser user) {
      this.userService.checkUserAllowed(user);
      this.userService.checkUserDataScope(user.getUserId());
      this.deptService.checkDeptDataScope(user.getDeptId());
      this.roleService.checkRoleDataScope(user.getRoleIds());
      if (!this.userService.checkUserNameUnique(user)) {
         return this.error("修改代理'" + user.getUserName() + "'失败，登录账号已存在");
      } else if (StringUtils.isNotEmpty(user.getPhonenumber()) && !this.userService.checkPhoneUnique(user)) {
         return this.error("修改代理'" + user.getUserName() + "'失败，手机号码已存在");
      } else if (StringUtils.isNotEmpty(user.getEmail()) && !this.userService.checkEmailUnique(user)) {
         return this.error("修改代理'" + user.getUserName() + "'失败，邮箱账号已存在");
      } else {
         LoginUser loginUser = this.tokenService.getLoginUser(ServletUtils.getRequest());
         SysUser obj = this.userService.selectUserById(user.getUserId());
         SysUser dealer = this.userService.selectUserById(obj.getDealerId());
         if (dealer.getServerLimit() < user.getServerLimit()) {
            Set<String> roles = loginUser.getRoles();
            return roles.contains("dealer") ? this.error("您当前最多可分配" + dealer.getServerLimit() + "个服务数量") : this.error("服务数量不能大于经销商的服务数量");
         } else {
            if (user.getParams().containsKey("templateIds")) {
               try {
                  List<Integer> templateIds = (List)user.getParams().get("templateIds");
                  this.templatePermsService.updateUserPerms((List)templateIds.stream().map(Integer::longValue).collect(Collectors.toList()), user.getUserId());
               } catch (Exception var6) {
               }
            }

            if (!Objects.equals(user.getServerLimit(), obj.getServerLimit())) {
               this.sysBussLogService.insertSysBussLog(BussLogEnum.OptServerNum.getType(), obj.getUserId(), obj.getUserName(), "服务数量变更 " + obj.getServerLimit() + " => " + user.getServerLimit());
            }

            user.setUpdateBy(SecurityUtils.getUsername());
            return this.toAjax(this.userService.updateUser(user));
         }
      }
   }

   @RequiresPermissions({"system:agent:remove"})
   @Log(
      title = "代理管理",
      businessType = BusinessType.DELETE
   )
   @DeleteMapping({"/{userIds}"})
   public AjaxResult remove(@PathVariable Long[] userIds) {
      if (ArrayUtils.contains(userIds, SecurityUtils.getUserId())) {
         return this.error("当前代理不能删除");
      } else {
         SysUser param = new SysUser();
         param.getParams().put("userIds", userIds);
         List<SysUser> users = this.userService.selectUserList(param);
         int num = this.userService.deleteUserByIds(userIds);
         Iterator var5 = users.iterator();

         while(var5.hasNext()) {
            SysUser user = (SysUser)var5.next();
            this.sysBussLogService.insertSysBussLog(BussLogEnum.DelAgent.getType(), user.getUserId(), user.getUserName(), "删除代理 " + user.getUserName());
         }

         return this.toAjax(num);
      }
   }

   @RequiresPermissions({"system:agent:resetPwd"})
   @Log(
      title = "代理管理",
      businessType = BusinessType.UPDATE
   )
   @PutMapping({"/resetPwd"})
   public AjaxResult resetPwd(@RequestBody SysUser user) {
      this.userService.checkUserAllowed(user);
      this.userService.checkUserDataScope(user.getUserId());
      user.setUpdateBy(SecurityUtils.getUsername());
      return this.toAjax(this.userService.resetPwd(user));
   }

   @RequiresPermissions({"system:agent:resetPwd"})
   @Log(
      title = "代理管理",
      businessType = BusinessType.UPDATE
   )
   @PutMapping({"/changeStatus"})
   public AjaxResult changeStatus(@RequestBody SysUser user) {
      this.userService.checkUserAllowed(user);
      this.userService.checkUserDataScope(user.getUserId());
      user.setUpdateBy(SecurityUtils.getUsername());
      return this.toAjax(this.userService.updateUserStatus(user));
   }
}
